<?php
namespace TGYoshi;

use Exception;
use ManiaControl\Admin\AuthenticationManager;
use ManiaControl\Commands\CommandListener;
use ManiaControl\ManiaControl;
use ManiaControl\Players\Player;
use ManiaControl\Plugins\Plugin;

/**
 * Adds commands to load scripts or restarting the current one.
 *
 * @author TGYoshi
 */
class ScriptLoader implements Plugin, CommandListener {
	// Constants
	const SETTING_SCRIPTRELOADAUTHLEVEL = 'Auth level for the scriptreload command';
	const SETTING_SCRIPTLOADAUTHLEVEL = 'Auth level for the scriptload command';

	/** @var maniaControl $maniaControl */
	private $maniaControl = null;
	private $scriptsDir = null;

	/**
	 * Prepares the Plugin
	 *
	 * @param ManiaControl $maniaControl
	 * @return mixed
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		// Settings
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_SCRIPTRELOADAUTHLEVEL, AuthenticationManager::AUTH_LEVEL_ADMIN);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_SCRIPTLOADAUTHLEVEL, AuthenticationManager::AUTH_LEVEL_ADMIN);

		// Register for callbacks
		$this->maniaControl->getCommandManager()->registerCommandListener('scriptreload', $this, 'onCommandScriptReload', true);
		$this->maniaControl->getCommandManager()->registerCommandListener('reloadscript', $this, 'onCommandScriptReload', true);
		$this->maniaControl->getCommandManager()->registerCommandListener('scriptload', $this, 'onCommandScriptLoad', true);
		$this->maniaControl->getCommandManager()->registerCommandListener('loadscript', $this, 'onCommandScriptLoad', true);

		$scriptsDataDir = $maniaControl->getServer()->getDirectory()->getScriptsFolder();

		if ($maniaControl->getServer()->checkAccess($scriptsDataDir)) {
			$gameShort = $this->maniaControl->getMapManager()->getCurrentMap()->getGame();
			$game = '';
			switch ($gameShort) {
				case 'tm':
					$game = 'TrackMania';
					break;
				case 'sm':
					$game = 'ShootMania';
					break;
				case 'qm':
					$game = 'QuestMania';
					break;
			}
			if ($game != '') {
				$this->scriptsDir = $scriptsDataDir . 'Modes' . DIRECTORY_SEPARATOR . $game . DIRECTORY_SEPARATOR;
				// Final assertion
				if (!$maniaControl->getServer()->checkAccess($this->scriptsDir))
					$this->scriptsDir = null;
			}
		}

		return true;
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return 33;
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return 'ScriptLoader Plugin';
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return '5.0';
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return 'TGYoshi';
	}

	/**
	 *
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return 'The ScriptLoader plugin can reload the current script (useful for development) or load other scripts by issuing commands. Requires the controller to have access to the UserData directory.';
	}

	/**
	 * Handles the //scriptreload and //reloadscript commands.
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function onCommandScriptReload(array $chatCallback, Player $player) {
		$authLevel = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_SCRIPTRELOADAUTHLEVEL);
		if (!$this->maniaControl->getAuthenticationManager()->checkRight($player, $authLevel)) {
			$this->maniaControl->getAuthenticationManager()->sendNotAllowed($player);
			return;
		}

		$scriptNameArr = $this->maniaControl->getClient()->getScriptName();
		$scriptName = $scriptNameArr['CurrentValue'];

		// Workaround for a 'bug' in setModeScriptText.
		if ($scriptName === '<in-development>')
			$scriptName = $scriptNameArr['NextValue'];

		try {
			$this->loadScript($scriptName);
			$this->maniaControl->getChat()->sendSuccess("Reloaded script " . $scriptName . '!', $player->login);
		} catch (Exception $e) {
			$this->maniaControl->getChat()->sendError("Can not reload script: " . $e->getMessage(), $player->login);
		}
	}

	/**
	 * Handles the //scriptload and //loadscript commands.
	 *
	 * @param array  $chatCallback
	 * @param Player $player
	 */
	public function onCommandScriptLoad(array $chatCallback, Player $player) {
		$authLevel = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_SCRIPTLOADAUTHLEVEL);
		if (!$this->maniaControl->getAuthenticationManager()->checkRight($player, $authLevel)) {
			$this->maniaControl->getAuthenticationManager()->sendNotAllowed($player);
			return;
		}

		$params = explode(' ', $chatCallback[1][2], 2);
		if (count($params) === 2) {
			$scriptName = $params[1];

			// Append .Script.txt if left out
			if (strtolower(substr($scriptName, -11)) !== '.script.txt')
				$scriptName .= '.Script.txt';

			try {
				$this->loadScript($scriptName);
				$this->maniaControl->getChat()->sendSuccess("Loaded script " . $scriptName . '!', $player->login);
			} catch (Exception $e) {
				$this->maniaControl->getChat()->sendError("Can not reload script: " . $e->getMessage(), $player->login);
			}
		} else {
			$this->maniaControl->getChat()->sendError("Usage: /scriptload [scriptname], scriptname relative to the /Scripts/Modes/[game]/ folder.", $player->login);
		}
	}

	/**
	 * Loads a script if possible, otherwise throws an exception.
	 * $scriptName will be set to a proper capitalized etc. name.
	 *
	 * @param $scriptName
	 * @throws Exception
	 */
	private function loadScript(&$scriptName) {
		if ($this->scriptsDir !== null) {
			$scriptPath = $this->scriptsDir . $scriptName;

			if (!file_exists($scriptPath)) {
				throw new Exception('Script not found (' . $scriptPath . ').');
			}

			// Get 'real' script name (mainly nice for Windows)
			$scriptName = pathinfo(realpath($scriptPath))['basename'];

			$script = file_get_contents($scriptPath);
			$this->maniaControl->getClient()->setModeScriptText($script);
			$this->maniaControl->getClient()->setScriptName($scriptName);
		} else {
			throw new Exception('Scripts directory not found (' . $this->scriptsDir . ').');
		}
	}
}
